# -*- coding:utf-8 -*-
import requests
import json
import time
from baselib.log import file_logger

class Channel(object):

    DEBUG = True
    url_tmpl = ""
    app_path = ""
    use_ssl = False
    logger = file_logger()

    @classmethod
    def get(
            cls,
            app_path,
            params=None,
            headers=None,
            cookies=None,
            end_with_slash=True):
        return cls.send(
            app_path,
            params,
            headers,
            cookies,
            "GET",
            end_with_slash)

    @classmethod
    def post(
            cls,
            app_path,
            params=None,
            headers=None,
            cookies=None,
            end_with_slash=True):
        return cls.send(
            app_path,
            params,
            headers,
            cookies,
            "POST",
            end_with_slash)

    @classmethod
    def put(
            cls,
            app_path,
            params=None,
            headers=None,
            cookies=None,
            end_with_slash=True):
        return cls.send(
            app_path,
            params,
            headers,
            cookies,
            "PUT",
            end_with_slash)

    @classmethod
    def send(cls, app_path, params, headers, cookies, method, end_with_slash):
        url, params, headers, cookies = cls.format_input(
            app_path, params, headers, cookies, end_with_slash)
        if method.upper() in ['GET']:
            rsp = requests.get(
                url,
                params=params,
                headers=headers,
                cookies=cookies)
        elif method.upper() in ['POST']:
            rsp = requests.post(
                url,
                json=params,
                headers=headers,
                cookies=cookies)
        elif method.upper() in ['PUT']:
            rsp = requests.put(
                url,
                json=params,
                headers=headers,
                cookies=cookies)
        if not rsp.text:
            return {}
        time.sleep(0.3)
        return cls.format_output(rsp)

    @classmethod
    def common_headers(cls, headers, cookies):
        if headers is None:
            headers = {}
        if cookies is None:
            cookies = {}
        return headers, cookies

    @classmethod
    def format_input(
            cls,
            app_path,
            params,
            headers=None,
            cookies=None,
            end_with_slash=True):

        if params is None:
            params = {}
        url = cls.format_url(
            url=cls.url_tmpl.format(app_path=app_path),
            end_with_slash=end_with_slash
        )
        headers, cookies = cls.common_headers(headers, cookies)
        if cls.DEBUG:
            cls.logger.debug("[url] {}".format(url))
            cls.logger.debug("[params] {}".format(params))
            cls.logger.debug("[headers] {}".format(headers))
        return url, params, headers, cookies

    @classmethod
    def format_output(cls, rsp):
        data = json.loads(rsp.text)
        if isinstance(data, dict) and data.get("message", ""):
            raise RuntimeError(data)
        if cls.DEBUG:
            cls.logger.debug("[Response] {}".format(data))
        return data

    @classmethod
    def format_url(cls, url, use_ssl=None, end_with_slash=True):
        http = "http://"
        if use_ssl is None:
            use_ssl = cls.use_ssl
        if use_ssl:
            http = "https://"
        url = url.replace("https://", "").replace(
            "http://", "").replace("///", "/").replace("//", "/")
        if url.endswith("/"):
            url = url[:-1]
        if end_with_slash:
            url += "/"
        return http + url



# -*- coding:utf-8 -*-
from baselib.qywx import QyWechat
obj = QyWechat("")


class TestSplitMsgByLine:
    def test_normal(self):
        """ 测试正常的多行切割"""
        msg = "123456789\n"*3
        limit = 25
        expected = ["123456789\n123456789", "123456789"]
        assert obj.split_msg_by_line(msg, limit) == expected

    def test_long_line(self):
        """ 测试某一行的长度，超过limit """
        msg = "12345\n123456\n12345678901234567890123\n" * 3
        limit = 20
        excepted = ['12345\n123456', '12345678901234567...',
                    '12345\n123456', '12345678901234567...',
                    '12345\n123456', '12345678901234567...']
        assert obj.split_msg_by_line(msg, limit) == excepted

    def test_limit_larger_than_msg_len(self):
        msg = "hello\nworld"
        limit = 20
        expected = ["hello\nworld"]
        assert obj.split_msg_by_line(msg, limit) == expected

    def test_empty_msg(self):
        msg = ""
        limit = 10
        expected = [""]
        assert obj.split_msg_by_line(msg, limit) == expected

    def test_single_char_limit(self):
        msg = "hello\nworld"
        limit = 10
        expected = ['hello', 'world']
        assert obj.split_msg_by_line(msg, limit) == expected


class TestCutMsg:
    def test_cut_msg_within_limit(self):
        msg = "Hello, World!"
        assert obj.cut_msg(msg, limit=20) == msg

    def test_cut_msg_exceed_limit(self):
        msg = "This is a very long message that exceeds the limit of 20 characters."
        assert obj.cut_msg(msg, limit=26) == "This is a very long me\n..."

    def test_cut_msg_with_custom_post_msg(self):
        msg = "This is a very long message that exceeds the limit of 20 characters."
        assert obj.cut_msg(msg, limit=31, post_msg=" [truncated]") == "This is a very long [truncated]"

    def test_cut_msg_with_unicode_characters(self):
        msg = "你好，世界！"
        assert obj.cut_msg(msg, limit=20) == msg

    def test_cut_msg_with_emoji(self):
        msg = "This is a message with an emoji 😊"
        assert obj.cut_msg(msg, limit=25) == "This is a message wit\n..."


class TestSplitMsg:
    test_obj = QyWechat("")

    def test_split_msg_normal(self):
        msg = "hello world"
        limit = 5
        expected = ["hello", " worl", "d"]
        assert self.test_obj.split_msg(msg, limit) == expected

    def test_split_msg_single_page(self):
        msg = "hello world"
        limit = 20
        expected = ["hello world"]
        assert self.test_obj.split_msg(msg, limit) == expected

    def test_split_msg_empty_msg(self):
        msg = ""
        limit = 10
        expected = [""]
        assert self.test_obj.split_msg(msg, limit) == expected

    def test_split_msg_limit_larger_than_msg_len(self):
        msg = "hello world"
        limit = 50
        expected = ["hello world"]
        assert self.test_obj.split_msg(msg, limit) == expected

    def test_split_msg_unicode(self):
        msg = "你好，世界"
        limit = 9
        expected = ["你好，", "世界"]
        assert self.test_obj.split_msg(msg, limit) == expected


# -*- coding: utf-8 -*-
"""
企业微信机器人消息类封装
- 消息类型：https://developer.work.weixin.qq.com/document/path/90236
- 加解密方案说明: https://developer.work.weixin.qq.com/devtool/introduce?id=36388
@author     : coreylin
@createTime : 2018/12/22
"""
import json
import base64
import hashlib

from baselib.py import urllib2
from baselib.py import IS_PY3

if not IS_PY3:
    import ssl
    ssl._create_default_https_context = ssl._create_unverified_context


class QyWechat(object):
    """ 企业微信处理类
    """
    font_tmpl = '<font color="{color}">{info}</font>'

    class EnumPageType:
        # 按字符分页
        SPLIT = "split"
        # 按行分页
        SPLIT_BY_LINES = "split_by_lines"
        # 截断
        CUT = "cut"

    class EnumMsgType:
        MARKDOWN = "markdown"
        NEWS = "news"
        TEXT = "text"
        IMAGE = "image"

    def __init__(
            self,
            key,
            proxy="",
            is_ssl=True,
            debug=False):
        # 在 init 中定义所有私有变量，方便查找，无特别意义
        self.key = None
        self.url = self.__proxy = None

        self.debug = debug
        self.set_proxy(proxy)
        self.set_ssl(is_ssl)
        self.update_key(key)

    def update_key(self, key):
        self.key = key
        self.url = "%s/cgi-bin/webhook/send?key=%s" % (self.domain, self.key)

    def set_ssl(self, is_ssl=False):
        if is_ssl:
            self.domain = "https://qyapi.weixin.qq.com"
        else:
            self.domain = "http://qyapi.weixin.qq.com"

    def set_proxy(self, proxy):
        self.__proxy = proxy

    def format_text(self, content, m_list=None, mm_list=None):
        """
        将输入文本转换为企业微信的请求格式
        :param content: 文本内容，最长不超过2048个字节，必须是utf8编码
        :param m_list:  userid的列表，提醒群中的指定成员(@某个成员)，
                        @all表示提醒所有人，如果开发者获取不到userid，可以使用mentioned_mobile_list
        :param mm_list: 手机号列表，提醒手机号对应的群成员(@某个成员)，@all表示提醒所有人
        :return:
        """
        if isinstance(m_list, str):
            m_list = [m_list]

        if isinstance(mm_list, str):
            mm_list = [mm_list]

        data = {
            "msgtype": "text",
            "text": {
                "content": content
            }
        }

        if m_list is not None and len(m_list) > 0:
            data["text"]["mentioned_list"] = m_list
        if mm_list is not None and len(mm_list) > 0:
            data["text"]["mentioned_mobile_list"] = m_list
        return data

    def format_image(self, base64_data, md5):
        data = {
            "msgtype": "image",
            "image": {
                "base64": base64_data,
                "md5": md5
            }
        }
        return data

    def format_news(self, title, target_url, desc=None, pic_url=None):
        data = {
            "msgtype": "news",
            "news": {
                "articles": [
                    {
                        "title": title,
                        "url": target_url
                    }
                ],

            }
        }
        if desc is not None:
            data["news"]["articles"][0]["description"] = desc
        if pic_url is not None:
            data["news"]["articles"][0]["picurl"] = pic_url
        return data

    def md5sum(self, filename):
        with open(filename, "rb") as f:
            fcont = f.read()
        f.close()
        fmd5 = hashlib.md5(fcont)
        return fmd5.hexdigest()

    def _send(self, data):
        """
        使用post方法，向企业微信发送消息
        :param data: 发送消息内容，为字典类型，通过get_xxx_data获得
        :return:企业微信返回的json消息，如：{u'errcode': 0, u'errmsg': u'ok'}
        """
        if len(self.__proxy) > 0:
            proxy = urllib2.ProxyHandler({'http': self.__proxy})
            opener = urllib2.build_opener(proxy)
            urllib2.install_opener(opener)
        post_json = json.dumps(data)
        if self.debug:
            print("url :{}, \ndata :{}".format(self.url, post_json))
            print("proxy : {}".format(self.__proxy))
        rsp = urllib2.urlopen(self.url, post_json.encode('utf-8')).read()
        if self.debug:
            print(rsp)
        return json.loads(rsp)

    def send_image(self, img_path):
        """ 发送图片
        :param img_path: 图片文件的本地路径
        :return: 企业微信返回的json消息，如：{u'errcode': 0, u'errmsg': u'ok'}
        """
        md5 = self.md5sum(img_path)
        with open(img_path, "rb") as f:
            # b64encode是编码，b64decode是解码
            base64_data = base64.b64encode(f.read())
        self._send(self.format_image(base64_data, md5))

    def send_news(self, data):
        pass

    def send_markdown(
            self,
            msg,
            page_type=EnumPageType.CUT,
            limit=4096,
            post_msg="\n...",
            to_user=None,
            to_party=None,
            to_tag=None,
            agent_id=None,
            enable_duplicate_check=None,
            duplicate_check_interval=None):
        """ 发送 markdown 格式的数据
        :param string msg: 发送的数据内容（markdown格式）
        :param int limit: 分页参数，每条消息的最大长度（企业微信默认4096），超过该长度报错
        :param string page_type: 分页参数，消息内容超长的处理方法， 取值范围：EnumPageType.CUT
        :param string post_msg: 分页参数，选择截断时，加在消息末尾的补充信息
        :param list to_user: 指定接收消息的成员，成员ID列表（多个接收者用‘|’分隔，最多支持1000个）
            特殊情况：指定为"@all"，则向该企业应用的全部成员发送
        :param list to_party:  指定接收消息的部门，部门ID列表，多个接收者用‘|’分隔，最多支持100个,
            当touser为"@all"时忽略本参数
        :param list to_tag: 指定接收消息的标签，标签ID列表，多个接收者用‘|’分隔，最多支持100个。
            当touser为"@all"时忽略本参数
        :param int agent_id: 企业应用的id，整型。企业内部开发，可在应用的设置页面查看；第三方服务商，可通过接口 获取企业授权信息 获取该参数值
        :param int enable_duplicate_check: 表示是否开启重复消息检查，0表示否，1表示是，默认0
        :param int duplicate_check_interval: 表示是否重复消息检查的时间间隔，默认1800s，最大不超过4小时
        :return:
        """
        # 根据分页方式，对页面数据进行剪切
        if page_type.lower() == self.EnumPageType.SPLIT:
            msg_list = self.split_msg(msg, limit)
        elif page_type.lower() == self.EnumPageType.SPLIT_BY_LINES:
            msg_list = self.split_msg_by_line(msg, limit)
        else:
            msg_list = [self.cut_msg(msg, limit, post_msg)]

        rsp_list = []
        for msg in msg_list:
            rsp_list.append(self.raw_send_markdown(
                msg, to_user, to_party, to_tag, agent_id,
                enable_duplicate_check, duplicate_check_interval))
        if len(rsp_list) == 1:
            return rsp_list[0]
        return rsp_list

    def raw_send_markdown(
            self,
            content,
            to_user=None,
            to_party=None,
            to_tag=None,
            agent_id=None,
            enable_duplicate_check=None,
            duplicate_check_interval=None):
        """ 发送 markdown 消息
        """
        data = {
            "msgtype": "markdown",
            "markdown": {
                "content": content
            }
        }
        if to_user is not None:
            data['touser'] = "|".join(to_user)
        if to_party is not None:
            data["toparty"] = " | ".join(to_party)
        if to_tag is not None:
            data["totag"] = " | ".join(to_tag)
        if agent_id is not None:
            data["agentid"] = agent_id
        if enable_duplicate_check is not None:
            data["enable_duplicate_check"] = enable_duplicate_check
        if duplicate_check_interval is not None:
            data["duplicate_check_interval"] = duplicate_check_interval
        return self._send(data)

    def send_text(self, data):
        pass

    def cut_msg(self, msg, limit=4096, post_msg="\n..."):
        """ 将 msg 长度限制在 limit 以内
            - 如果大于 limit， 则截断多余部分，并补充 post_msg
            - 如果小于等于 limit，则直接返回
        :param string msg: 消息内容
        :param int limit: 消息长度限制
        :param string post_msg: 截断末尾补充消息
        :rtype: string
        :return: 返回截断后数据
        """

        msg_unicode, encode = self.__decode_msg(msg)
        msg_len = len(msg_unicode.encode("utf-8"))
        if msg_len <= limit:
            return msg
        post_msg_unicode, _ = self.__decode_msg(post_msg)
        post_msg_len = len(post_msg_unicode.encode("utf-8"))
        idx = 0
        for idx in range(len(msg_unicode)):
            if len(msg_unicode[:-1 - idx].encode("utf-8")
                   ) <= limit - post_msg_len:
                break
        ret = msg_unicode[:-1 - idx] + post_msg_unicode
        return self.__encode_msg(ret, encode)

    def __decode_msg(self, msg):
        if IS_PY3:
            print(1, type(msg))
            return msg, type(msg)
        try:
            encode = "utf-8"
            msg_unicode = msg.decode("utf-8")
        except UnicodeEncodeError:
            encode = "unicode"
            msg_unicode = msg
        return msg_unicode, encode

    def __encode_msg(self, msg, encode):
        """ 将字符串 msg 转换成目标编码格式 encode
        :param msg:
        :param encode:
        :return:
        """
        # 兼容 python2
        if not IS_PY3:
            return msg.encode(encode)
        # 相同时无需转换
        if type(msg) == encode:
            return msg
        if type(msg) == str:
            return msg.encode()
        else:
            return msg.decode()

    def split_msg(self, msg, limit=4096):
        """ 将消息 msg 的长度限制（limit），拆分成多条消息(一条消息即一页）
        :param string msg: 消息内容
        :param int limit: 消息长度限制
        :rtype: list
        :return: 返回分页后的数据列表
        """
        msg_unicode, encode = self.__decode_msg(msg)
        msg_len = len(msg_unicode.encode("utf-8"))
        if msg_len <= limit:
            return [msg]
        lines = [""]
        line_no = 0
        for char in msg_unicode:
            if len(lines[line_no].encode("utf-8")) <= limit - \
                    len(char.encode("utf-8")):
                lines[line_no] += char
                continue
            lines.append(char)
            line_no += 1
        return [self.__encode_msg(i, encode) for i in lines]

    def split_msg_by_line(self, msg, limit=4096):
        """将数据按行分页

        本函数适用场景：消息分多行，单行不超过 limit 的情况，单行 limit 会直接截断

        :param str msg: 消息内容
        :param int limit: 消息长度限制
        :return: 返回分页后的数据列表
        :rtype: list
        """
        # 去除消息内容两端的空格
        msg = msg.strip()
        msg_unicode, encode = self.__decode_msg(msg)
        # 计算消息内容的字节数
        msg_len = len(msg_unicode.encode("utf-8"))
        # 如果消息内容的字节数小于限制，则直接返回
        if msg_len < limit:
            return [msg]
        # 初始化分页长度、页码、页列表
        page_len = page_no = 0
        page_list = [""]
        # 遍历消息内容的每一行
        for aline in msg_unicode.split("\n"):
            # 计算当前行的字节数
            len_aline = len("{}\n".format(aline).encode("utf-8"))
            if len_aline >= limit:
                page_len = 0
                page_no += 2
                page_list.append(self.cut_msg(aline, limit, "..."))
                page_list.append("")
                continue
            # 如果当前行加上分页长度小于限制，则将当前行添加到当前页
            if page_len + len_aline <= limit:
                page_len += len_aline
                page_list[page_no] += "{}\n".format(aline)
                continue
            # 如果当前行加上分页长度大于限制，则将当前行添加到新的一页
            page_len = 0
            page_no += 1
            page_list.append(aline)
        # 返回分页后的数据列表
        return [self.__encode_msg(i.strip(), encode) for i in page_list if len(i) > 0]


if __name__ == "__main__":
    def demo():
        key = ""  # robot key
        robot = QyWechat(key=key, is_ssl=True, debug=True)
        aline = "[hello](http://www.baidu.com)" * 100
        print(robot.send_markdown(
            "# test\n" + aline + "\n" + aline + "\n",
            page_type=robot.EnumPageType.SPLIT_BY_LINES))
    # demo()
    robot = QyWechat("")
    msg = "你好，世界"
    limit = 9
    expected = ["你好，", "世界"]
    print(type(msg))
    print(type(robot.split_msg(msg, limit)[0]))
    print(type(expected[0]))



# -*- coding:utf-8 -*- 
import logging
import logging.handlers


def file_logger(log_path="runtime.log", log_level="DEBUG", log_format=None,
                time_rotating='D', encoding='utf-8', name=None):
    """
    创建一个文件日志记录器

    :param log_path: str, 日志文件路径，默认为"runtime.log"
    :param log_level: str, 日志级别，ERROR/WARNING/INFO/DEBUG， 默认为"DEBUG"
    :param log_format: str, 每条日志输出格式，默认为"[%(asctime)s][%(levelname)s][%(filename)s:%(lineno)d-%(module)s] %(message)s"
    :param time_rotating: str, 日志切割时间，默认为"D"，可选值: S(秒)/M(分)/H(小时)/D(天)/W(周)/midnight(深夜)
    :param encoding: str, 编码，默认为"utf-8"
    :param name: str, 日志记录器名称，默认为当前模块名

    :return: logger, 日志记录器对象
    """
    # 如果没有传入name参数，则使用当前模块名
    if not name:
        name = __name__
    # 创建一个日志记录器对象
    logger = logging.getLogger(name)
    # 设置日志记录器的日志级别
    logger.setLevel(log_level.upper())

    # 如果没有传入log_format参数，则使用默认的日志输出格式
    if log_format is None:
        log_format = '[%(asctime)s][%(levelname)s][%(filename)s:%(lineno)d-%(module)s] %(message)s'
    # 创建一个日志输出格式对象
    formatter = logging.Formatter(log_format)

    # 如果传入了time_rotating参数，则创建一个按时间切割的日志文件处理器
    if time_rotating:
        file_handler = logging.handlers.TimedRotatingFileHandler(
            log_path, when=time_rotating, encoding=encoding)
    # 否则创建一个普通的日志文件处理器
    else:
        file_handler = logging.FileHandler(
            log_path, encoding=encoding)

    # 设置日志文件处理器的日志级别和输出格式
    file_handler.setLevel(level=log_level)
    file_handler.setFormatter(formatter)
    # 将日志文件处理器添加到日志记录器中
    logger.addHandler(file_handler)

    # 返回日志记录器对象
    return logger


if __name__ == "__main__":
    logger = file_logger()
    logger.info("info")
    logger.debug("debug")


# -*- coding:utf-8 -*-
from baselib.time import Period
from datetime import datetime, timedelta


def test_get_recent_hour():
    """  测试获取近N个小时
    """
    # 测试点1：验证 小时的起止时间
    for i in [1, 2, 5, 10, 24, 25]:
        start_time, end_time = Period.get_recent_hour(i)
        now = datetime.now()
        del_time = end_time - start_time
        assert del_time.total_seconds() - 3600 * i < 0.1
    # print(end_time, now)
    del_time = now - end_time
    # print(del_time)
    assert del_time.total_seconds() < 1

    # 测试点2：验证截止统计时间
    start_time, end_time = Period.get_recent_hour(1, 10)
    now = datetime.now()
    del_time = end_time - start_time
    assert abs(del_time.total_seconds() - 3600) < 0.1
    del_time = now - end_time
    # print(now, end_time, del_time)
    assert abs(del_time.total_seconds() - 600) < 1


def test_get_recent_day():
    start_time, end_time = Period.get_recent_day(1)
    now = datetime.now()
    del_time = end_time - start_time

    assert abs(del_time.total_seconds() - 3600 * 24) < 0.1
    del_time = end_time - now
    # print(del_time.total_seconds())
    assert abs(del_time.total_seconds()) < 1
    # print(start_time, end_time)


def test_get_day_begin_end():
    start_time = Period.get_day_begin()
    end_time = Period.get_day_end()
    now = Period.now()
    assert start_time.day == now.day and \
        start_time.month == now.month and \
        start_time.year == now.year and \
        start_time.hour == 0 and \
        start_time.minute == 0 and \
        start_time.second == 0
    assert end_time.day == now.day and \
        end_time.month == now.month and \
        end_time.year == now.year and \
        end_time.hour == 23 and \
        end_time.minute == 59 and \
        end_time.second == 59

    yestorday, _ = Period.get_recent_day(1)
    start_time = Period.get_day_begin(yestorday)
    end_time = Period.get_day_end(yestorday)
    assert start_time.day == yestorday.day and \
        start_time.month == yestorday.month and \
        start_time.year == yestorday.year and \
        start_time.hour == 0 and \
        start_time.minute == 0 and \
        start_time.second == 0
    assert end_time.day == yestorday.day and \
        end_time.month == yestorday.month and \
        end_time.year == yestorday.year and \
        end_time.hour == 23 and \
        end_time.minute == 59 and \
        end_time.second == 59


def test_format_datetime():
    now = Period.get_day_end()
    assert Period.format_datetime(now) == now.strftime("%Y-%m-%d %H:%M:%S")
    assert Period.format_t_datetime(now) == now.strftime("%Y-%m-%dT%H:%M:%SZ")
    assert Period.format_time(now) == now.strftime("%H:%M:%S")
    assert Period.format_date(now) == now.strftime("%Y-%m-%d")



import os
from baselib.json import JsonFile


class TestJsonFile(object):

    def test_json_file(self):
        filename = "test.json"
        if os.path.isfile(filename):
            os.remove(filename)
        data = JsonFile.read(filename)
        assert data == {}
        JsonFile.write(filename, {"msg": "hello"})
        data = JsonFile.read(filename)
        assert data == {"msg": "hello"}
        JsonFile.update(filename, {"msg": "world"})
        data = JsonFile.read(filename)
        assert data == {"msg": "world"}
        JsonFile.update(filename, {"info": "hello"})
        data = JsonFile.read(filename)
        assert data == {"info": "hello", "msg": "world"}

        # 验证非合法json 读取
        os.remove(filename)
        with open(filename, "w") as fid:
            fid.write("test")
        data = JsonFile.read(filename)
        assert data == {}

        # 验证中文读写
        data = {"msg": "中文", "msg2": u"中文"}
        JsonFile.write(filename, data)
        data = JsonFile.read(filename)
        assert data['msg'] == "中文"
        assert data['msg2'] == "中文"




"""
https://open.feishu.cn/document/client-docs/bot-v3/add-custom-bot#f62e72d5
"""
import requests
import json


class FeishuRobot(object):

    def __init__(self, token, debug=True):
        self.token = token
        self.url = f"https://open.feishu.cn/open-apis/bot/v2/hook/{token}"
        self.debug = debug

    def _send(self, data: dict, headers: dict = None):
        headers = self.common_headers(headers)
        if self.debug:
            print(f'[url]{self.url}')
            print(f'[headers]{headers}')
            print(f'[data]{data}')
        response = requests.post(self.url, headers=headers, json=data)
        # if response.status_code == 200:
        #     print("消息发送成功")
        # else:
        #     print("消息发送失败")
        if self.debug:
            print(f"[rsp]{response.text}")
        return json.loads(str(response.text))

    def common_headers(self, headers=None):
        if headers is None:
            headers = {}
        headers['Content-Type'] = headers.get(
            'Content-Type', 'application/json')
        return headers

    def get_username_by_id(self, user_id):
        return user_id

    def send_text(self, content, at_list=None):
        """ 发送文本消息
        :param content:
        :param at_list:
        :return:
        """
        if at_list is None:
            at_list = []
        at_msg_list = []
        for user_id in at_list:
            user_name = self.get_username_by_id(user_id)
            at_msg_list.append(f'<at user_id="{user_id}>{user_name}</at>"')
        at_msg_msg = ",".join(at_msg_list)
        data = {
            "msg_type": "text",
            "content": {
                "text": f"{content}{at_msg_msg}"
            }
        }
        return self._send(data)

    def send_post(self, title, content):
        """ 发送富文本
        :param title:
        :param content: 二维字典列表，富文本消息内容。由多个段落组成，每个段落为一个[]节点，其中包含若干个节点。
            content = [
                [
                    dict(tag="text", text="项目有更新:"),
                    dict(tag="a", text="请查看", href="http://www.example.com/"),
                    dict(tag="at", user_id="oui_xxxxxx")
                ],
                [],
            ]
            上述的标签tag，支持：
            - text： 文本
            - a: 超链接
            - img: 图片
            - at: @相关人
        :return:
        """
        if content and not isinstance(content[0], list):
            content = [content]
        data = {
            "msg_type": "post",
            "content": {
                "post": {
                    "zh_cn": {
                        "title": title,
                        "content": content
                    }
                }
            }
        }
        return self._send(data)

    def send_share_chat(self, share_chat_id):
        """ 发送群名片
        :param share_chat_id: 群 ID。获取方式请参见 群 ID 说明。
        :return:
        """
        data = {
            "msg_type": "share_chat",
            "content": {
                "share_chat_id": share_chat_id
            }
        }
        return self._send(data)

    def send_img(self, image_key):
        """ 发送图片
        :param image_key: 图片Key。可通过 上传图片 接口获取 image_key。
        :return:
        """
        data = {
            "msg_type": "image",
            "content": {
                "image_key": image_key
            }
        }
        return self._send(data)

    def send_card(self, card):
        """
        :param card: 可以通过消息卡片工具生成卡片数据库
            https://open.feishu.cn/tool/cardbuilder?templateId=ctp_AA1o9hqhLG0u
        :return:
        """
        data = {
            "msg_type": "interactive",
            "card": card
        }
        return self._send(data)


if __name__ == "__main__":
    token = ""
    robot = FeishuRobot(token=token)
    content = [
        dict(tag="text", text="项目有更新:"),
        dict(tag="a", text="请查看", href="http://www.example.com/"),
        dict(tag="at", user_id="oui_xxxxxx")
    ]
    card = {
        "config": {
            "wide_screen_mode": True
        },
        "elements": [
            {
                "tag": "markdown",
                "content": "zheg"
            },
            {
                "alt": {
                    "content": "",
                    "tag": "plain_text"
                },
                "img_key": "img_v2_041b28e3-5680-48c2-9af2-497ace79333g",
                "tag": "img",
                "mode": "crop_center",
                "compact_width": True
            },
            {
                "tag": "action",
                "actions": [
                    {
                        "tag": "button",
                        "text": {
                            "tag": "plain_text",
                            "content": "这是跳转按钮"
                        },
                        "type": "default",
                        "multi_url": {
                            "url": "https://feishu.cn",
                            "pc_url": "",
                            "android_url": "",
                            "ios_url": ""
                        },
                        "value": {
                            "sss": "fdsa"
                        }
                    }
                ]
            }
        ],
        "header": {
            "template": "blue",
            "title": {
                "content": "xxxxx",
                "tag": "plain_text"
            }
        }
    }
    print(robot.send_card(card))
